// change_samplerate.c : Count number of rx_callbacks before and after the
//                       sample rate is changed (single tuner)

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "sdrplay_api.h"

sdrplay_api_DeviceT *chosenDevice = NULL;
long nStreamACallbacks;
long nStreamBCallbacks;
long nEventCallbacks;

void StreamACallback(short *xi, short *xq, sdrplay_api_StreamCbParamsT *params, unsigned int numSamples, unsigned int reset, void *cbContext)
{
    nStreamACallbacks++;
    return;
}

void StreamBCallback(short *xi, short *xq, sdrplay_api_StreamCbParamsT *params, unsigned int numSamples, unsigned int reset, void *cbContext)
{
    nStreamBCallbacks++;
    return;
}

void EventCallback(sdrplay_api_EventT eventId, sdrplay_api_TunerSelectT tuner, sdrplay_api_EventParamsT *params, void *cbContext)
{
    nEventCallbacks++;
    switch(eventId)
    {
    case sdrplay_api_PowerOverloadChange:
        sdrplay_api_Update(chosenDevice->dev, tuner, sdrplay_api_Update_Ctrl_OverloadMsgAck);
        break;
    default:
        break;
    }
}

int main(int argc, char *argv[])
{
    sdrplay_api_DeviceT devs[6];
    unsigned int ndev;
    int i;
    float ver = 0.0;
    sdrplay_api_ErrT err;
    sdrplay_api_DeviceParamsT *deviceParams = NULL;
    sdrplay_api_CallbackFnsT cbFns;
    sdrplay_api_RxChannelParamsT *chParams;

    double fs1 = 2685312.0;
    double fs2 = 2685313.0;

    if (argc == 3)
    {
        sscanf(argv[1], "%lf", &fs1);
        sscanf(argv[2], "%lf", &fs2);
    }
    printf("fs1: %lf\n", fs1);
    printf("fs2: %lf\n", fs2);
    printf("\n");

    unsigned int chosenIdx = 0;

    // Open API
    if ((err = sdrplay_api_Open()) != sdrplay_api_Success)
    {
        printf("sdrplay_api_Open failed %s\n", sdrplay_api_GetErrorString(err));
    }
    else
    {
        // Enable debug logging output
        if ((err = sdrplay_api_DebugEnable(NULL, 1)) != sdrplay_api_Success)
        {
            printf("sdrplay_api_DebugEnable failed %s\n", sdrplay_api_GetErrorString(err));
        }

        // Check API versions match
        if ((err = sdrplay_api_ApiVersion(&ver)) != sdrplay_api_Success)
        {
            printf("sdrplay_api_ApiVersion failed %s\n", sdrplay_api_GetErrorString(err));
        }
        if (ver != SDRPLAY_API_VERSION)
        {
            printf("API version don't match (local=%.2f dll=%.2f)\n", SDRPLAY_API_VERSION, ver);
            goto CloseApi;
        }

        // Lock API while device selection is performed
        sdrplay_api_LockDeviceApi();

        // Fetch list of available devices
        if ((err = sdrplay_api_GetDevices(devs, &ndev, sizeof(devs) /
               sizeof(sdrplay_api_DeviceT))) != sdrplay_api_Success)
        {
            printf("sdrplay_api_GetDevices failed %s\n", sdrplay_api_GetErrorString(err));
            goto UnlockDeviceAndCloseApi;
        }
        printf("MaxDevs=%ld NumDevs=%d\n", sizeof(devs) / sizeof(sdrplay_api_DeviceT), ndev);
        if (ndev > 0)
        {
            for (i = 0; i < (int)ndev; i++)
            {
                if (devs[i].hwVer == SDRPLAY_RSPduo_ID)
                    printf("Dev%d: SerNo=%s hwVer=%d tuner=0x%.2x rspDuoMode=0x%.2x\n", i,
                       devs[i].SerNo, devs[i].hwVer , devs[i].tuner, devs[i].rspDuoMode);
                else
                    printf("Dev%d: SerNo=%s hwVer=%d tuner=0x%.2x\n", i, devs[i].SerNo,
                       devs[i].hwVer, devs[i].tuner);
            }

            // Pick first device of any type
            for (i = 0; i < (int)ndev; i++)
            {
                chosenIdx = i;
                break;
            }
            if (i == ndev)
            {
                printf("Couldn't find a suitable device to open - exiting\n");
                goto UnlockDeviceAndCloseApi;
            }
            printf("chosenDevice = %d\n", chosenIdx);
            chosenDevice = &devs[chosenIdx];

            // Select chosen device
            if ((err = sdrplay_api_SelectDevice(chosenDevice)) != sdrplay_api_Success)
            {
                printf("sdrplay_api_SelectDevice failed %s\n", sdrplay_api_GetErrorString(err));
                goto UnlockDeviceAndCloseApi;
            }

            // Unlock API now that device is selected
            sdrplay_api_UnlockDeviceApi();

            // Retrieve device parameters so they can be changed if wanted
            if ((err = sdrplay_api_GetDeviceParams(chosenDevice->dev, &deviceParams)) !=
               sdrplay_api_Success)
            {
                printf("sdrplay_api_GetDeviceParams failed %s\n",
                      sdrplay_api_GetErrorString(err));
                goto CloseApi;
            }

            // Check for NULL pointers before changing settings
            if (deviceParams == NULL)
            {
                printf("sdrplay_api_GetDeviceParams returned NULL deviceParams pointer\n");
                goto CloseApi;
            }

            // Configure dev parameters
            if (deviceParams->devParams != NULL)
            {
                // Change from default Fs to fs1
                deviceParams->devParams->fsFreq.fsHz = fs1;
            }

            // Configure tuner parameters
            chParams = deviceParams->rxChannelA;
            if (chParams != NULL)
            {
                chParams->tunerParams.rfFreq.rfHz = 1010000.0;
                chParams->tunerParams.bwType = sdrplay_api_BW_1_536;
                chParams->tunerParams.ifType = sdrplay_api_IF_Zero;
                chParams->tunerParams.gain.gRdB = 40;
                chParams->tunerParams.gain.LNAstate = 5;

                // Disable AGC
                chParams->ctrlParams.agc.enable = sdrplay_api_AGC_DISABLE;

                // Disable decimation
                chParams->ctrlParams.decimation.enable = 0;
                chParams->ctrlParams.decimation.decimationFactor = 1;
            }
            else
            {
                printf("sdrplay_api_GetDeviceParams returned NULL chParams pointer\n");
                goto CloseApi;
            }

            // Assign callback functions to be passed to sdrplay_api_Init()
            cbFns.StreamACbFn = StreamACallback;
            cbFns.StreamBCbFn = StreamBCallback;
            cbFns.EventCbFn = EventCallback;

            // Now we're ready to start by calling the initialisation function
            // This will configure the device and start streaming
            if ((err = sdrplay_api_Init(chosenDevice->dev, &cbFns, NULL)) != sdrplay_api_Success)
            {
                printf("sdrplay_api_Init failed %s\n", sdrplay_api_GetErrorString(err));
                goto CloseApi;
            }

            nStreamACallbacks = 0L;
            nStreamBCallbacks = 0L;
            nEventCallbacks = 0L;

            // run for about 2 seconds
            sleep(2);

            printf("callbacks at sample rate = fs1\n");
            printf("    nStreamACallbacks=%ld\n", nStreamACallbacks);
            printf("    nStreamBCallbacks=%ld\n", nStreamBCallbacks);
            printf("    nEventCallbacks=%ld\n", nEventCallbacks);

            // Change Fs from fs1 to fs2
            deviceParams->devParams->fsFreq.fsHz = fs2;
            // Disable decimation
            chParams->ctrlParams.decimation.enable = 0;
            chParams->ctrlParams.decimation.decimationFactor = 1;

            sdrplay_api_Update(chosenDevice->dev, chosenDevice->tuner,
                (sdrplay_api_ReasonForUpdateT) (sdrplay_api_Update_Dev_Fs |
                 sdrplay_api_Update_Ctrl_Decimation));

            nStreamACallbacks = 0L;
            nStreamBCallbacks = 0L;
            nEventCallbacks = 0L;

            // run for about 2 seconds
            sleep(2);

            printf("callbacks at sample rate = fs2\n");
            printf("    nStreamACallbacks=%ld\n", nStreamACallbacks);
            printf("    nStreamBCallbacks=%ld\n", nStreamBCallbacks);
            printf("    nEventCallbacks=%ld\n", nEventCallbacks);

            // Finished with device so uninitialise it
            if ((err = sdrplay_api_Uninit(chosenDevice->dev)) != sdrplay_api_Success)
            {
                printf("sdrplay_api_Uninit failed %s\n", sdrplay_api_GetErrorString(err));
                goto CloseApi;
            }

            // Release device (make it available to other applications)
            sdrplay_api_ReleaseDevice(chosenDevice);

        }
UnlockDeviceAndCloseApi:
        // Unlock API
        sdrplay_api_UnlockDeviceApi();

CloseApi:
        // Close API
        sdrplay_api_Close();
    }
    return 0;
}
